#pragma once
#include <vector>
#include "types.h"

//--Parameters--:

// No constants i.e. can be adjusted at runtime...
extern float std_alpha; //4.0f
extern float std_sigma; //0.03f
extern float std_beta; //20
extern int std_gamma; //UNDEFINED, set before using local removal. You can find an optimal parameter by testing different parameters on training data

//--Helper Functions--:

// d(b,c) from Equation 1+2 in paper
float distance4D_2 ( rect r1, rect r2 );
//  Attraction between two boxes (Part of Equation 3)
float attraction ( rect r1, rect r2, float sigma = std_sigma );
// For trajectory optimization (Part of Equation 7).
float normalizedAttraction ( rect r1, rect r2, float sigma = std_sigma );
// Equation 3 without tracker weights set and Equation 5 with tracker weights set. Can also return single attractions.
float calculateAttraction ( trackList &list, rect r, int frame, std::vector<int> idxlist, std::vector<float> trackerWeights = std::vector<float>(), std::vector<float> & singleAttractions = * ( ( std::vector<float>* ) 0 ) );
// Fusion without gradient descent. Can also return the individual attractions required by trajectory optimization
int findTrackingRectWithGreatestAttraction ( trackList &list, int frame, std::vector<int> idxlist = std::vector<int>(), std::vector<float> trackerWeights = std::vector<float>(),std::vector<float> & attractions = * ( ( std::vector<float>* ) 0 ) );
//Equation 12:
std::vector<float> calculateTrackerPerformances ( trackList &list, std::vector<int> idxlist = std::vector<int>(), std::vector<float> trackerWeights = std::vector<float>() );

//--Approach--:

// Basic/Weighted Approach (leave "trackerWeights" empty for basic approach):
rect performFusionForFrame ( trackList & list, int frame, std::vector<int> idxlist = std::vector<int>(), std::vector<float> trackerWeights = std::vector<float>(), bool gradient_ascent = true );
void performFusionForAllFrames ( trackList & list, track& fusionResult, std::vector<int> idxlist = std::vector<int>(), std::vector<float> weights = std::vector<float>(), bool gradient_ascent = true );

// Get tracker weights:
std::vector<float> calculateTrackerWeights ( trackData & data,trackList &annotation );

//Trajectory Optimization. Note that the interface of the function only supports offline tracking. However, it can internally simulate online tracking:
void performFusionForAllFrames_TrajectoryOptimization ( trackList & list, track & fusionResult, std::vector<int> idxlist = std::vector<int>(), std::vector<float> trackerWeights = std::vector<float>(), bool online = false, bool gradientAscent = true );

//Local Removal. Set std_gamma before using function.
std::vector<int> localRemoval ( trackList & list, std::vector<int> idxlist = std::vector<int>(), std::vector<float> weights = std::vector<float>() );

//Global Removal. Supports currently only WEIGHTED and TRAJECTORY_OPTIMIZATION approaches as base algorithms (last parameter):
std::vector <int> globalRemoval ( trackData & data,trackList &anno, std::vector<float> weights, bool useTrajectoryOptimization = true );

//--Configurations used in the paper--:

enum fusionApproach
{
    BASIC,WEIGHTED,TRAJECTORY_OPTIMIZATION,LOCAL_REMOVAL,GLOBAL_REMOVAL,TRAJECTORY_OPTIMIZATION_ONLINE, // Configurations used in the paper. 
    TRAJECTORY_OPTIMIZATION_NO_TRAINING,TRAJECTORY_OPTIMIZATION_NO_TRAINING_ONLINE, LOCAL_REMOVAL_NO_TRAINING // Can be used if no training data is available
};
  
inline void performFusionApproach ( trackList & list, track & fusionResult, fusionApproach approach, trackData & trainingData =  * ( ( trackData* ) 0 ), trackList & annotation  =  * ( ( trackList* ) 0 ) )
{
    switch ( approach )
    {
    case BASIC:
        performFusionForAllFrames ( list,fusionResult );
        break;
    case WEIGHTED:
        assert ( ( &trainingData ) && ( &annotation ) );
        performFusionForAllFrames ( list,fusionResult, std::vector<int>(), calculateTrackerWeights ( trainingData,annotation ) );
        break;
    case TRAJECTORY_OPTIMIZATION:
        assert ( ( &trainingData ) && ( &annotation ) );
        performFusionForAllFrames_TrajectoryOptimization ( list,fusionResult, std::vector<int>(), calculateTrackerWeights ( trainingData,annotation ) );
        break;
    case TRAJECTORY_OPTIMIZATION_ONLINE:
        assert ( ( &trainingData ) && ( &annotation ) );
        performFusionForAllFrames_TrajectoryOptimization ( list,fusionResult, std::vector<int>(), calculateTrackerWeights ( trainingData,annotation ),true );
        break;
    case LOCAL_REMOVAL:
        assert ( ( &trainingData ) && ( &annotation ) );
        {
            std::vector<float> weights = calculateTrackerWeights ( trainingData,annotation );
            performFusionForAllFrames_TrajectoryOptimization ( list,fusionResult, localRemoval ( list, std::vector<int>(), weights ), weights, false );
        }
        break;
    case GLOBAL_REMOVAL:
        assert ( ( &trainingData ) && ( &annotation ) );
        {
            std::vector<float> weights = calculateTrackerWeights ( trainingData,annotation );
            performFusionForAllFrames_TrajectoryOptimization ( list,fusionResult, globalRemoval ( trainingData,annotation,weights ), calculateTrackerWeights ( trainingData,annotation ),false );
        }
        break;
    case TRAJECTORY_OPTIMIZATION_NO_TRAINING:
        performFusionForAllFrames_TrajectoryOptimization ( list,fusionResult );
        break;
    case TRAJECTORY_OPTIMIZATION_NO_TRAINING_ONLINE:
        performFusionForAllFrames_TrajectoryOptimization ( list,fusionResult, std::vector<int>(), std::vector<float>(),true );
        break;
    case LOCAL_REMOVAL_NO_TRAINING:
        performFusionForAllFrames_TrajectoryOptimization ( list,fusionResult, localRemoval (list));

    }
}
